/*
 * The Relaxer artifact
 * Copyright (c) 2000-2004, ASAMI Tomoharu, All rights reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 *
 * - Redistributions of source code must retain the above copyright
 *   notice, this list of conditions and the following disclaimer. 
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
 * LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
 * OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */
package com.netpotlet.jetty;

import java.io.*;
import java.io.IOException;
import java.io.PrintWriter;
import java.io.Writer;
import java.net.URL;
import javax.xml.parsers.*;
import org.w3c.dom.*;
import org.xml.sax.*;

/**
 * <b>Keyboard</b> is generated from compact-keyboard.rng by Relaxer.
 * This class is derived from:
 * 
 * <!-- for programmer
 * <element name="keyboard">
 *       <element name="display"><ref name="position"/></element>
 *       <zeroOrMore>
 *         <element name="key">
 *           <ref name="position"/>
 *           <optional><ref name="size"/></optional>
 *           <oneOrMore><ref name="label"/></oneOrMore>
 *         </element>
 *       </zeroOrMore>
 *     </element>-->
 * <!-- for javadoc -->
 * <pre> &lt;element name="keyboard"&gt;
 *       &lt;element name="display"&gt;&lt;ref name="position"/&gt;&lt;/element&gt;
 *       &lt;zeroOrMore&gt;
 *         &lt;element name="key"&gt;
 *           &lt;ref name="position"/&gt;
 *           &lt;optional&gt;&lt;ref name="size"/&gt;&lt;/optional&gt;
 *           &lt;oneOrMore&gt;&lt;ref name="label"/&gt;&lt;/oneOrMore&gt;
 *         &lt;/element&gt;
 *       &lt;/zeroOrMore&gt;
 *     &lt;/element&gt;</pre>
 *
 * @version compact-keyboard.rng (Sun Feb 29 22:05:36 JST 2004)
 * @author  Relaxer 1.1b (http://www.relaxer.org)
 */
public class Keyboard implements java.io.Serializable, Cloneable, IRVisitable, IRNode {
    private KeyboardDisplay keyboardDisplay_;
    // List<KeyboardKey>
    private java.util.List keyboardKey_ = new java.util.ArrayList();
    private IRNode parentRNode_;

    /**
     * Creates a <code>Keyboard</code>.
     *
     */
    public Keyboard() {
    }

    /**
     * Creates a <code>Keyboard</code>.
     *
     * @param source
     */
    public Keyboard(Keyboard source) {
        setup(source);
    }

    /**
     * Creates a <code>Keyboard</code> by the Stack <code>stack</code>
     * that contains Elements.
     * This constructor is supposed to be used internally
     * by the Relaxer system.
     *
     * @param stack
     */
    public Keyboard(RStack stack) {
        setup(stack);
    }

    /**
     * Creates a <code>Keyboard</code> by the Document <code>doc</code>.
     *
     * @param doc
     */
    public Keyboard(Document doc) {
        setup(doc.getDocumentElement());
    }

    /**
     * Creates a <code>Keyboard</code> by the Element <code>element</code>.
     *
     * @param element
     */
    public Keyboard(Element element) {
        setup(element);
    }

    /**
     * Creates a <code>Keyboard</code> by the File <code>file</code>.
     *
     * @param file
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public Keyboard(File file) throws IOException, SAXException, ParserConfigurationException {
        setup(file);
    }

    /**
     * Creates a <code>Keyboard</code>
     * by the String representation of URI <code>uri</code>.
     *
     * @param uri
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public Keyboard(String uri) throws IOException, SAXException, ParserConfigurationException {
        setup(uri);
    }

    /**
     * Creates a <code>Keyboard</code> by the URL <code>url</code>.
     *
     * @param url
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public Keyboard(URL url) throws IOException, SAXException, ParserConfigurationException {
        setup(url);
    }

    /**
     * Creates a <code>Keyboard</code> by the InputStream <code>in</code>.
     *
     * @param in
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public Keyboard(InputStream in) throws IOException, SAXException, ParserConfigurationException {
        setup(in);
    }

    /**
     * Creates a <code>Keyboard</code> by the InputSource <code>is</code>.
     *
     * @param is
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public Keyboard(InputSource is) throws IOException, SAXException, ParserConfigurationException {
        setup(is);
    }

    /**
     * Creates a <code>Keyboard</code> by the Reader <code>reader</code>.
     *
     * @param reader
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public Keyboard(Reader reader) throws IOException, SAXException, ParserConfigurationException {
        setup(reader);
    }

    /**
     * Initializes the <code>Keyboard</code> by the Keyboard <code>source</code>.
     *
     * @param source
     */
    public void setup(Keyboard source) {
        int size;
        if (source.keyboardDisplay_ != null) {
            setKeyboardDisplay((KeyboardDisplay)source.getKeyboardDisplay().clone());
        }
        this.keyboardKey_.clear();
        size = source.keyboardKey_.size();
        for (int i = 0;i < size;i++) {
            addKeyboardKey((KeyboardKey)source.getKeyboardKey(i).clone());
        }
    }

    /**
     * Initializes the <code>Keyboard</code> by the Document <code>doc</code>.
     *
     * @param doc
     */
    public void setup(Document doc) {
        setup(doc.getDocumentElement());
    }

    /**
     * Initializes the <code>Keyboard</code> by the Element <code>element</code>.
     *
     * @param element
     */
    public void setup(Element element) {
        init(element);
    }

    /**
     * Initializes the <code>Keyboard</code> by the Stack <code>stack</code>
     * that contains Elements.
     * This constructor is supposed to be used internally
     * by the Relaxer system.
     *
     * @param stack
     */
    public void setup(RStack stack) {
        init(stack.popElement());
    }

    /**
     * @param element
     */
    private void init(Element element) {
        RStack stack = new RStack(element);
        setKeyboardDisplay(new KeyboardDisplay(stack));
        keyboardKey_.clear();
        while (true) {
            if (KeyboardKey.isMatch(stack)) {
                addKeyboardKey(new KeyboardKey(stack));
            } else {
                break;
            }
        }
    }

    /**
     * @return Object
     */
    public Object clone() {
        return (new Keyboard(this));
    }

    /**
     * Creates a DOM representation of the object.
     * Result is appended to the Node <code>parent</code>.
     *
     * @param parent
     */
    public void makeElement(Node parent) {
        Document doc;
        if (parent instanceof Document) {
            doc = (Document)parent;
        } else {
            doc = parent.getOwnerDocument();
        }
        Element element = doc.createElement("keyboard");
        int size;
        this.keyboardDisplay_.makeElement(element);
        size = this.keyboardKey_.size();
        for (int i = 0;i < size;i++) {
            KeyboardKey value = (KeyboardKey)this.keyboardKey_.get(i);
            value.makeElement(element);
        }
        parent.appendChild(element);
    }

    /**
     * Initializes the <code>Keyboard</code> by the File <code>file</code>.
     *
     * @param file
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public void setup(File file) throws IOException, SAXException, ParserConfigurationException {
        setup(file.toURL());
    }

    /**
     * Initializes the <code>Keyboard</code>
     * by the String representation of URI <code>uri</code>.
     *
     * @param uri
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public void setup(String uri) throws IOException, SAXException, ParserConfigurationException {
        setup(UJAXP.getDocument(uri, UJAXP.FLAG_NONE));
    }

    /**
     * Initializes the <code>Keyboard</code> by the URL <code>url</code>.
     *
     * @param url
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public void setup(URL url) throws IOException, SAXException, ParserConfigurationException {
        setup(UJAXP.getDocument(url, UJAXP.FLAG_NONE));
    }

    /**
     * Initializes the <code>Keyboard</code> by the InputStream <code>in</code>.
     *
     * @param in
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public void setup(InputStream in) throws IOException, SAXException, ParserConfigurationException {
        setup(UJAXP.getDocument(in, UJAXP.FLAG_NONE));
    }

    /**
     * Initializes the <code>Keyboard</code> by the InputSource <code>is</code>.
     *
     * @param is
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public void setup(InputSource is) throws IOException, SAXException, ParserConfigurationException {
        setup(UJAXP.getDocument(is, UJAXP.FLAG_NONE));
    }

    /**
     * Initializes the <code>Keyboard</code> by the Reader <code>reader</code>.
     *
     * @param reader
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public void setup(Reader reader) throws IOException, SAXException, ParserConfigurationException {
        setup(UJAXP.getDocument(reader, UJAXP.FLAG_NONE));
    }

    /**
     * Creates a DOM document representation of the object.
     *
     * @exception ParserConfigurationException
     * @return Document
     */
    public Document makeDocument() throws ParserConfigurationException {
        Document doc = UJAXP.makeDocument();
        makeElement(doc);
        return (doc);
    }

    /**
     * Gets the KeyboardDisplay property <b>keyboardDisplay</b>.
     *
     * @return KeyboardDisplay
     */
    public KeyboardDisplay getKeyboardDisplay() {
        return (keyboardDisplay_);
    }

    /**
     * Sets the KeyboardDisplay property <b>keyboardDisplay</b>.
     *
     * @param keyboardDisplay
     */
    public void setKeyboardDisplay(KeyboardDisplay keyboardDisplay) {
        this.keyboardDisplay_ = keyboardDisplay;
        if (keyboardDisplay != null) {
            keyboardDisplay.rSetParentRNode(this);
        }
    }

    /**
     * Gets the KeyboardKey property <b>keyboardKey</b>.
     *
     * @return KeyboardKey[]
     */
    public KeyboardKey[] getKeyboardKey() {
        KeyboardKey[] array = new KeyboardKey[keyboardKey_.size()];
        return ((KeyboardKey[])keyboardKey_.toArray(array));
    }

    /**
     * Sets the KeyboardKey property <b>keyboardKey</b>.
     *
     * @param keyboardKey
     */
    public void setKeyboardKey(KeyboardKey[] keyboardKey) {
        this.keyboardKey_.clear();
        for (int i = 0;i < keyboardKey.length;i++) {
            addKeyboardKey(keyboardKey[i]);
        }
        for (int i = 0;i < keyboardKey.length;i++) {
            keyboardKey[i].rSetParentRNode(this);
        }
    }

    /**
     * Sets the KeyboardKey property <b>keyboardKey</b>.
     *
     * @param keyboardKey
     */
    public void setKeyboardKey(KeyboardKey keyboardKey) {
        this.keyboardKey_.clear();
        addKeyboardKey(keyboardKey);
        if (keyboardKey != null) {
            keyboardKey.rSetParentRNode(this);
        }
    }

    /**
     * Adds the KeyboardKey property <b>keyboardKey</b>.
     *
     * @param keyboardKey
     */
    public void addKeyboardKey(KeyboardKey keyboardKey) {
        this.keyboardKey_.add(keyboardKey);
        if (keyboardKey != null) {
            keyboardKey.rSetParentRNode(this);
        }
    }

    /**
     * Adds the KeyboardKey property <b>keyboardKey</b>.
     *
     * @param keyboardKey
     */
    public void addKeyboardKey(KeyboardKey[] keyboardKey) {
        for (int i = 0;i < keyboardKey.length;i++) {
            addKeyboardKey(keyboardKey[i]);
        }
        for (int i = 0;i < keyboardKey.length;i++) {
            keyboardKey[i].rSetParentRNode(this);
        }
    }

    /**
     * Gets number of the KeyboardKey property <b>keyboardKey</b>.
     *
     * @return int
     */
    public int sizeKeyboardKey() {
        return (keyboardKey_.size());
    }

    /**
     * Gets the KeyboardKey property <b>keyboardKey</b> by index.
     *
     * @param index
     * @return KeyboardKey
     */
    public KeyboardKey getKeyboardKey(int index) {
        return ((KeyboardKey)keyboardKey_.get(index));
    }

    /**
     * Sets the KeyboardKey property <b>keyboardKey</b> by index.
     *
     * @param index
     * @param keyboardKey
     */
    public void setKeyboardKey(int index, KeyboardKey keyboardKey) {
        this.keyboardKey_.set(index, keyboardKey);
        if (keyboardKey != null) {
            keyboardKey.rSetParentRNode(this);
        }
    }

    /**
     * Adds the KeyboardKey property <b>keyboardKey</b> by index.
     *
     * @param index
     * @param keyboardKey
     */
    public void addKeyboardKey(int index, KeyboardKey keyboardKey) {
        this.keyboardKey_.add(index, keyboardKey);
        if (keyboardKey != null) {
            keyboardKey.rSetParentRNode(this);
        }
    }

    /**
     * Remove the KeyboardKey property <b>keyboardKey</b> by index.
     *
     * @param index
     */
    public void removeKeyboardKey(int index) {
        this.keyboardKey_.remove(index);
    }

    /**
     * Remove the KeyboardKey property <b>keyboardKey</b> by object.
     *
     * @param keyboardKey
     */
    public void removeKeyboardKey(KeyboardKey keyboardKey) {
        this.keyboardKey_.remove(keyboardKey);
    }

    /**
     * Clear the KeyboardKey property <b>keyboardKey</b>.
     *
     */
    public void clearKeyboardKey() {
        this.keyboardKey_.clear();
    }

    /**
     * Makes an XML text representation.
     *
     * @return String
     */
    public String makeTextDocument() {
        StringBuffer buffer = new StringBuffer();
        makeTextElement(buffer);
        return (new String(buffer));
    }

    /**
     * Makes an XML text representation.
     *
     * @param buffer
     */
    public void makeTextElement(StringBuffer buffer) {
        int size;
        buffer.append("<keyboard");
        buffer.append(">");
        keyboardDisplay_.makeTextElement(buffer);
        size = this.keyboardKey_.size();
        for (int i = 0;i < size;i++) {
            KeyboardKey value = (KeyboardKey)this.keyboardKey_.get(i);
            value.makeTextElement(buffer);
        }
        buffer.append("</keyboard>");
    }

    /**
     * Makes an XML text representation.
     *
     * @param buffer
     * @exception IOException
     */
    public void makeTextElement(Writer buffer) throws IOException {
        int size;
        buffer.write("<keyboard");
        buffer.write(">");
        keyboardDisplay_.makeTextElement(buffer);
        size = this.keyboardKey_.size();
        for (int i = 0;i < size;i++) {
            KeyboardKey value = (KeyboardKey)this.keyboardKey_.get(i);
            value.makeTextElement(buffer);
        }
        buffer.write("</keyboard>");
    }

    /**
     * Makes an XML text representation.
     *
     * @param buffer
     */
    public void makeTextElement(PrintWriter buffer) {
        int size;
        buffer.print("<keyboard");
        buffer.print(">");
        keyboardDisplay_.makeTextElement(buffer);
        size = this.keyboardKey_.size();
        for (int i = 0;i < size;i++) {
            KeyboardKey value = (KeyboardKey)this.keyboardKey_.get(i);
            value.makeTextElement(buffer);
        }
        buffer.print("</keyboard>");
    }

    /**
     * Makes an XML text representation.
     *
     * @param buffer
     */
    public void makeTextAttribute(StringBuffer buffer) {
    }

    /**
     * Makes an XML text representation.
     *
     * @param buffer
     * @exception IOException
     */
    public void makeTextAttribute(Writer buffer) throws IOException {
    }

    /**
     * Makes an XML text representation.
     *
     * @param buffer
     */
    public void makeTextAttribute(PrintWriter buffer) {
    }

    /**
     * Returns a String representation of this object.
     * While this method informs as XML format representaion, 
     *  it's purpose is just information, not making 
     * a rigid XML documentation.
     *
     * @return String
     */
    public String toString() {
        try {
            return (makeTextDocument());
        } catch (Exception e) {
            return (super.toString());
        }
    }

    /**
     * Accepts the Visitor for enter behavior.
     *
     * @param visitor
     * @return boolean
     */
    public boolean enter(IRVisitor visitor) {
        return (visitor.enter(this));
    }

    /**
     * Accepts the Visitor for leave behavior.
     *
     * @param visitor
     */
    public void leave(IRVisitor visitor) {
        visitor.leave(this);
    }

    /**
     * Gets the IRNode property <b>parentRNode</b>.
     *
     * @return IRNode
     */
    public IRNode rGetParentRNode() {
        return (parentRNode_);
    }

    /**
     * Sets the IRNode property <b>parentRNode</b>.
     *
     * @param parentRNode
     */
    public void rSetParentRNode(IRNode parentRNode) {
        this.parentRNode_ = parentRNode;
    }

    /**
     * Gets child RNodes.
     *
     * @return IRNode[]
     */
    public IRNode[] rGetRNodes() {
        java.util.List classNodes = new java.util.ArrayList();
        if (keyboardDisplay_ != null) {
            classNodes.add(keyboardDisplay_);
        }
        classNodes.addAll(keyboardKey_);
        IRNode[] nodes = new IRNode[classNodes.size()];
        return ((IRNode[])classNodes.toArray(nodes));
    }

    /**
     * Tests if a Element <code>element</code> is valid
     * for the <code>Keyboard</code>.
     *
     * @param element
     * @return boolean
     */
    public static boolean isMatch(Element element) {
        if (!URelaxer.isTargetElement(element, "keyboard")) {
            return (false);
        }
        RStack target = new RStack(element);
        boolean $match$ = false;
        Element child;
        if (!KeyboardDisplay.isMatchHungry(target)) {
            return (false);
        }
        $match$ = true;
        while (true) {
            if (!KeyboardKey.isMatchHungry(target)) {
                break;
            }
            $match$ = true;
        }
        if (!target.isEmptyElement()) {
            return (false);
        }
        return (true);
    }

    /**
     * Tests if elements contained in a Stack <code>stack</code>
     * is valid for the <code>Keyboard</code>.
     * This mehtod is supposed to be used internally
     * by the Relaxer system.
     *
     * @param stack
     * @return boolean
     */
    public static boolean isMatch(RStack stack) {
        Element element = stack.peekElement();
        if (element == null) {
            return (false);
        }
        return (isMatch(element));
    }

    /**
     * Tests if elements contained in a Stack <code>stack</code>
     * is valid for the <code>Keyboard</code>.
     * This method consumes the stack contents during matching operation.
     * This mehtod is supposed to be used internally
     * by the Relaxer system.
     *
     * @param stack
     * @return boolean
     */
    public static boolean isMatchHungry(RStack stack) {
        Element element = stack.peekElement();
        if (element == null) {
            return (false);
        }
        if (isMatch(element)) {
            stack.popElement();
            return (true);
        } else {
            return (false);
        }
    }
}
