/*
 * The Relaxer artifact
 * Copyright (c) 2000-2004, ASAMI Tomoharu, All rights reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 *
 * - Redistributions of source code must retain the above copyright
 *   notice, this list of conditions and the following disclaimer. 
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
 * LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
 * OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */
package com.netpotlet.jetty;

import java.io.*;
import java.io.IOException;
import java.io.PrintWriter;
import java.io.Writer;
import java.net.URL;
import javax.xml.parsers.*;
import org.w3c.dom.*;
import org.xml.sax.*;

/**
 * <b>KeyboardKey</b> is generated from compact-keyboard.rng by Relaxer.
 * This class is derived from:
 * 
 * <!-- for programmer
 * <element name="key">
 *           <ref name="position"/>
 *           <optional><ref name="size"/></optional>
 *           <oneOrMore><ref name="label"/></oneOrMore>
 *         </element>-->
 * <!-- for javadoc -->
 * <pre> &lt;element name="key"&gt;
 *           &lt;ref name="position"/&gt;
 *           &lt;optional&gt;&lt;ref name="size"/&gt;&lt;/optional&gt;
 *           &lt;oneOrMore&gt;&lt;ref name="label"/&gt;&lt;/oneOrMore&gt;
 *         &lt;/element&gt;</pre>
 *
 * @version compact-keyboard.rng (Sun Feb 29 22:05:36 JST 2004)
 * @author  Relaxer 1.1b (http://www.relaxer.org)
 */
public class KeyboardKey implements java.io.Serializable, Cloneable, IRVisitable, IRNode {
    private Position position_;
    private ISizeChoice size_;
    // List<Label>
    private java.util.List label_ = new java.util.ArrayList();
    private IRNode parentRNode_;

    /**
     * Creates a <code>KeyboardKey</code>.
     *
     */
    public KeyboardKey() {
    }

    /**
     * Creates a <code>KeyboardKey</code>.
     *
     * @param source
     */
    public KeyboardKey(KeyboardKey source) {
        setup(source);
    }

    /**
     * Creates a <code>KeyboardKey</code> by the Stack <code>stack</code>
     * that contains Elements.
     * This constructor is supposed to be used internally
     * by the Relaxer system.
     *
     * @param stack
     */
    public KeyboardKey(RStack stack) {
        setup(stack);
    }

    /**
     * Creates a <code>KeyboardKey</code> by the Document <code>doc</code>.
     *
     * @param doc
     */
    public KeyboardKey(Document doc) {
        setup(doc.getDocumentElement());
    }

    /**
     * Creates a <code>KeyboardKey</code> by the Element <code>element</code>.
     *
     * @param element
     */
    public KeyboardKey(Element element) {
        setup(element);
    }

    /**
     * Creates a <code>KeyboardKey</code> by the File <code>file</code>.
     *
     * @param file
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public KeyboardKey(File file) throws IOException, SAXException, ParserConfigurationException {
        setup(file);
    }

    /**
     * Creates a <code>KeyboardKey</code>
     * by the String representation of URI <code>uri</code>.
     *
     * @param uri
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public KeyboardKey(String uri) throws IOException, SAXException, ParserConfigurationException {
        setup(uri);
    }

    /**
     * Creates a <code>KeyboardKey</code> by the URL <code>url</code>.
     *
     * @param url
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public KeyboardKey(URL url) throws IOException, SAXException, ParserConfigurationException {
        setup(url);
    }

    /**
     * Creates a <code>KeyboardKey</code> by the InputStream <code>in</code>.
     *
     * @param in
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public KeyboardKey(InputStream in) throws IOException, SAXException, ParserConfigurationException {
        setup(in);
    }

    /**
     * Creates a <code>KeyboardKey</code> by the InputSource <code>is</code>.
     *
     * @param is
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public KeyboardKey(InputSource is) throws IOException, SAXException, ParserConfigurationException {
        setup(is);
    }

    /**
     * Creates a <code>KeyboardKey</code> by the Reader <code>reader</code>.
     *
     * @param reader
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public KeyboardKey(Reader reader) throws IOException, SAXException, ParserConfigurationException {
        setup(reader);
    }

    /**
     * Initializes the <code>KeyboardKey</code> by the KeyboardKey <code>source</code>.
     *
     * @param source
     */
    public void setup(KeyboardKey source) {
        int size;
        if (source.position_ != null) {
            setPosition((Position)source.getPosition().clone());
        }
        if (source.size_ != null) {
            setSize((ISizeChoice)source.getSize().clone());
        }
        this.label_.clear();
        size = source.label_.size();
        for (int i = 0;i < size;i++) {
            addLabel((Label)source.getLabel(i).clone());
        }
    }

    /**
     * Initializes the <code>KeyboardKey</code> by the Document <code>doc</code>.
     *
     * @param doc
     */
    public void setup(Document doc) {
        setup(doc.getDocumentElement());
    }

    /**
     * Initializes the <code>KeyboardKey</code> by the Element <code>element</code>.
     *
     * @param element
     */
    public void setup(Element element) {
        init(element);
    }

    /**
     * Initializes the <code>KeyboardKey</code> by the Stack <code>stack</code>
     * that contains Elements.
     * This constructor is supposed to be used internally
     * by the Relaxer system.
     *
     * @param stack
     */
    public void setup(RStack stack) {
        init(stack.popElement());
    }

    /**
     * @param element
     */
    private void init(Element element) {
        RStack stack = new RStack(element);
        setPosition(new Position(stack));
        if (SizeSize2.isMatch(stack)) {
            setSize(new SizeSize2(stack));
        } else if (SizeSize1.isMatch(stack)) {
            setSize(new SizeSize1(stack));
        } else {
        }
        label_.clear();
        while (true) {
            if (Label.isMatch(stack)) {
                addLabel(new Label(stack));
            } else {
                break;
            }
        }
    }

    /**
     * @return Object
     */
    public Object clone() {
        return (new KeyboardKey(this));
    }

    /**
     * Creates a DOM representation of the object.
     * Result is appended to the Node <code>parent</code>.
     *
     * @param parent
     */
    public void makeElement(Node parent) {
        Document doc;
        if (parent instanceof Document) {
            doc = (Document)parent;
        } else {
            doc = parent.getOwnerDocument();
        }
        Element element = doc.createElement("key");
        int size;
        this.position_.makeElement(element);
        if (this.size_ != null) {
            this.size_.makeElement(element);
        }
        size = this.label_.size();
        for (int i = 0;i < size;i++) {
            Label value = (Label)this.label_.get(i);
            value.makeElement(element);
        }
        parent.appendChild(element);
    }

    /**
     * Initializes the <code>KeyboardKey</code> by the File <code>file</code>.
     *
     * @param file
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public void setup(File file) throws IOException, SAXException, ParserConfigurationException {
        setup(file.toURL());
    }

    /**
     * Initializes the <code>KeyboardKey</code>
     * by the String representation of URI <code>uri</code>.
     *
     * @param uri
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public void setup(String uri) throws IOException, SAXException, ParserConfigurationException {
        setup(UJAXP.getDocument(uri, UJAXP.FLAG_NONE));
    }

    /**
     * Initializes the <code>KeyboardKey</code> by the URL <code>url</code>.
     *
     * @param url
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public void setup(URL url) throws IOException, SAXException, ParserConfigurationException {
        setup(UJAXP.getDocument(url, UJAXP.FLAG_NONE));
    }

    /**
     * Initializes the <code>KeyboardKey</code> by the InputStream <code>in</code>.
     *
     * @param in
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public void setup(InputStream in) throws IOException, SAXException, ParserConfigurationException {
        setup(UJAXP.getDocument(in, UJAXP.FLAG_NONE));
    }

    /**
     * Initializes the <code>KeyboardKey</code> by the InputSource <code>is</code>.
     *
     * @param is
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public void setup(InputSource is) throws IOException, SAXException, ParserConfigurationException {
        setup(UJAXP.getDocument(is, UJAXP.FLAG_NONE));
    }

    /**
     * Initializes the <code>KeyboardKey</code> by the Reader <code>reader</code>.
     *
     * @param reader
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public void setup(Reader reader) throws IOException, SAXException, ParserConfigurationException {
        setup(UJAXP.getDocument(reader, UJAXP.FLAG_NONE));
    }

    /**
     * Creates a DOM document representation of the object.
     *
     * @exception ParserConfigurationException
     * @return Document
     */
    public Document makeDocument() throws ParserConfigurationException {
        Document doc = UJAXP.makeDocument();
        makeElement(doc);
        return (doc);
    }

    /**
     * Gets the Position property <b>position</b>.
     *
     * @return Position
     */
    public Position getPosition() {
        return (position_);
    }

    /**
     * Sets the Position property <b>position</b>.
     *
     * @param position
     */
    public void setPosition(Position position) {
        this.position_ = position;
        if (position != null) {
            position.rSetParentRNode(this);
        }
    }

    /**
     * Gets the ISizeChoice property <b>size</b>.
     *
     * @return ISizeChoice
     */
    public ISizeChoice getSize() {
        return (size_);
    }

    /**
     * Sets the ISizeChoice property <b>size</b>.
     *
     * @param sizeValue
     */
    public void setSize(ISizeChoice sizeValue) {
        this.size_ = sizeValue;
        if (sizeValue != null) {
            sizeValue.rSetParentRNode(this);
        }
    }

    /**
     * Gets the Label property <b>label</b>.
     *
     * @return Label[]
     */
    public Label[] getLabel() {
        Label[] array = new Label[label_.size()];
        return ((Label[])label_.toArray(array));
    }

    /**
     * Sets the Label property <b>label</b>.
     *
     * @param label
     */
    public void setLabel(Label[] label) {
        this.label_.clear();
        for (int i = 0;i < label.length;i++) {
            addLabel(label[i]);
        }
        for (int i = 0;i < label.length;i++) {
            label[i].rSetParentRNode(this);
        }
    }

    /**
     * Sets the Label property <b>label</b>.
     *
     * @param label
     */
    public void setLabel(Label label) {
        this.label_.clear();
        addLabel(label);
        if (label != null) {
            label.rSetParentRNode(this);
        }
    }

    /**
     * Adds the Label property <b>label</b>.
     *
     * @param label
     */
    public void addLabel(Label label) {
        this.label_.add(label);
        if (label != null) {
            label.rSetParentRNode(this);
        }
    }

    /**
     * Adds the Label property <b>label</b>.
     *
     * @param label
     */
    public void addLabel(Label[] label) {
        for (int i = 0;i < label.length;i++) {
            addLabel(label[i]);
        }
        for (int i = 0;i < label.length;i++) {
            label[i].rSetParentRNode(this);
        }
    }

    /**
     * Gets number of the Label property <b>label</b>.
     *
     * @return int
     */
    public int sizeLabel() {
        return (label_.size());
    }

    /**
     * Gets the Label property <b>label</b> by index.
     *
     * @param index
     * @return Label
     */
    public Label getLabel(int index) {
        return ((Label)label_.get(index));
    }

    /**
     * Sets the Label property <b>label</b> by index.
     *
     * @param index
     * @param label
     */
    public void setLabel(int index, Label label) {
        this.label_.set(index, label);
        if (label != null) {
            label.rSetParentRNode(this);
        }
    }

    /**
     * Adds the Label property <b>label</b> by index.
     *
     * @param index
     * @param label
     */
    public void addLabel(int index, Label label) {
        this.label_.add(index, label);
        if (label != null) {
            label.rSetParentRNode(this);
        }
    }

    /**
     * Remove the Label property <b>label</b> by index.
     *
     * @param index
     */
    public void removeLabel(int index) {
        this.label_.remove(index);
    }

    /**
     * Remove the Label property <b>label</b> by object.
     *
     * @param label
     */
    public void removeLabel(Label label) {
        this.label_.remove(label);
    }

    /**
     * Clear the Label property <b>label</b>.
     *
     */
    public void clearLabel() {
        this.label_.clear();
    }

    /**
     * Makes an XML text representation.
     *
     * @return String
     */
    public String makeTextDocument() {
        StringBuffer buffer = new StringBuffer();
        makeTextElement(buffer);
        return (new String(buffer));
    }

    /**
     * Makes an XML text representation.
     *
     * @param buffer
     */
    public void makeTextElement(StringBuffer buffer) {
        int size;
        buffer.append("<key");
        if (size_ != null) {
            size_.makeTextAttribute(buffer);
        }
        buffer.append(">");
        position_.makeTextElement(buffer);
        if (size_ != null) {
            size_.makeTextElement(buffer);
        }
        size = this.label_.size();
        for (int i = 0;i < size;i++) {
            Label value = (Label)this.label_.get(i);
            value.makeTextElement(buffer);
        }
        buffer.append("</key>");
    }

    /**
     * Makes an XML text representation.
     *
     * @param buffer
     * @exception IOException
     */
    public void makeTextElement(Writer buffer) throws IOException {
        int size;
        buffer.write("<key");
        if (size_ != null) {
            size_.makeTextAttribute(buffer);
        }
        buffer.write(">");
        position_.makeTextElement(buffer);
        if (size_ != null) {
            size_.makeTextElement(buffer);
        }
        size = this.label_.size();
        for (int i = 0;i < size;i++) {
            Label value = (Label)this.label_.get(i);
            value.makeTextElement(buffer);
        }
        buffer.write("</key>");
    }

    /**
     * Makes an XML text representation.
     *
     * @param buffer
     */
    public void makeTextElement(PrintWriter buffer) {
        int size;
        buffer.print("<key");
        if (size_ != null) {
            size_.makeTextAttribute(buffer);
        }
        buffer.print(">");
        position_.makeTextElement(buffer);
        if (size_ != null) {
            size_.makeTextElement(buffer);
        }
        size = this.label_.size();
        for (int i = 0;i < size;i++) {
            Label value = (Label)this.label_.get(i);
            value.makeTextElement(buffer);
        }
        buffer.print("</key>");
    }

    /**
     * Makes an XML text representation.
     *
     * @param buffer
     */
    public void makeTextAttribute(StringBuffer buffer) {
    }

    /**
     * Makes an XML text representation.
     *
     * @param buffer
     * @exception IOException
     */
    public void makeTextAttribute(Writer buffer) throws IOException {
    }

    /**
     * Makes an XML text representation.
     *
     * @param buffer
     */
    public void makeTextAttribute(PrintWriter buffer) {
    }

    /**
     * Returns a String representation of this object.
     * While this method informs as XML format representaion, 
     *  it's purpose is just information, not making 
     * a rigid XML documentation.
     *
     * @return String
     */
    public String toString() {
        try {
            return (makeTextDocument());
        } catch (Exception e) {
            return (super.toString());
        }
    }

    /**
     * Accepts the Visitor for enter behavior.
     *
     * @param visitor
     * @return boolean
     */
    public boolean enter(IRVisitor visitor) {
        return (visitor.enter(this));
    }

    /**
     * Accepts the Visitor for leave behavior.
     *
     * @param visitor
     */
    public void leave(IRVisitor visitor) {
        visitor.leave(this);
    }

    /**
     * Gets the IRNode property <b>parentRNode</b>.
     *
     * @return IRNode
     */
    public IRNode rGetParentRNode() {
        return (parentRNode_);
    }

    /**
     * Sets the IRNode property <b>parentRNode</b>.
     *
     * @param parentRNode
     */
    public void rSetParentRNode(IRNode parentRNode) {
        this.parentRNode_ = parentRNode;
    }

    /**
     * Gets child RNodes.
     *
     * @return IRNode[]
     */
    public IRNode[] rGetRNodes() {
        java.util.List classNodes = new java.util.ArrayList();
        if (position_ != null) {
            classNodes.add(position_);
        }
        if (size_ != null) {
            classNodes.add(size_);
        }
        classNodes.addAll(label_);
        IRNode[] nodes = new IRNode[classNodes.size()];
        return ((IRNode[])classNodes.toArray(nodes));
    }

    /**
     * Tests if a Element <code>element</code> is valid
     * for the <code>KeyboardKey</code>.
     *
     * @param element
     * @return boolean
     */
    public static boolean isMatch(Element element) {
        if (!URelaxer.isTargetElement(element, "key")) {
            return (false);
        }
        RStack target = new RStack(element);
        boolean $match$ = false;
        Element child;
        if (!Position.isMatchHungry(target)) {
            return (false);
        }
        $match$ = true;
        if (SizeSize2.isMatchHungry(target)) {
            $match$ = true;
        } else if (SizeSize1.isMatchHungry(target)) {
            $match$ = true;
        } else {
        }
        if (!Label.isMatchHungry(target)) {
            return (false);
        }
        $match$ = true;
        while (true) {
            if (!Label.isMatchHungry(target)) {
                break;
            }
            $match$ = true;
        }
        if (!target.isEmptyElement()) {
            return (false);
        }
        return (true);
    }

    /**
     * Tests if elements contained in a Stack <code>stack</code>
     * is valid for the <code>KeyboardKey</code>.
     * This mehtod is supposed to be used internally
     * by the Relaxer system.
     *
     * @param stack
     * @return boolean
     */
    public static boolean isMatch(RStack stack) {
        Element element = stack.peekElement();
        if (element == null) {
            return (false);
        }
        return (isMatch(element));
    }

    /**
     * Tests if elements contained in a Stack <code>stack</code>
     * is valid for the <code>KeyboardKey</code>.
     * This method consumes the stack contents during matching operation.
     * This mehtod is supposed to be used internally
     * by the Relaxer system.
     *
     * @param stack
     * @return boolean
     */
    public static boolean isMatchHungry(RStack stack) {
        Element element = stack.peekElement();
        if (element == null) {
            return (false);
        }
        if (isMatch(element)) {
            stack.popElement();
            return (true);
        } else {
            return (false);
        }
    }
}
