/*
 * The Relaxer artifact
 * Copyright (c) 2000-2004, ASAMI Tomoharu, All rights reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 *
 * - Redistributions of source code must retain the above copyright
 *   notice, this list of conditions and the following disclaimer. 
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
 * LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
 * OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */
package com.netpotlet.jetty;

import java.io.*;
import java.io.IOException;
import java.io.PrintWriter;
import java.io.Writer;
import java.net.URL;
import javax.xml.parsers.*;
import org.w3c.dom.*;
import org.xml.sax.*;

/**
 * <b>Label</b> is generated from compact-keyboard.rng by Relaxer.
 * This class is derived from:
 * 
 * <!-- for programmer
 * <element name="label">
 *       <optional>
 *         <attribute name="default">
 *           <data datatypeLibrary="http://www.w3.org/2001/XMLSchema-datatypes" type="boolean"/>
 *         </attribute>
 *       </optional>
 *       <choice>
 *         <attribute name="type">
 *           <ref name="type.list"/>
 *         </attribute>
 *         <element name="type">
 *           <ref name="type.list"/>
 *         </element>
 *       </choice>
 *       <text/>
 *     </element>-->
 * <!-- for javadoc -->
 * <pre> &lt;element name="label"&gt;
 *       &lt;optional&gt;
 *         &lt;attribute name="default"&gt;
 *           &lt;data datatypeLibrary="http://www.w3.org/2001/XMLSchema-datatypes" type="boolean"/&gt;
 *         &lt;/attribute&gt;
 *       &lt;/optional&gt;
 *       &lt;choice&gt;
 *         &lt;attribute name="type"&gt;
 *           &lt;ref name="type.list"/&gt;
 *         &lt;/attribute&gt;
 *         &lt;element name="type"&gt;
 *           &lt;ref name="type.list"/&gt;
 *         &lt;/element&gt;
 *       &lt;/choice&gt;
 *       &lt;text/&gt;
 *     &lt;/element&gt;</pre>
 *
 * @version compact-keyboard.rng (Sun Feb 29 22:05:36 JST 2004)
 * @author  Relaxer 1.1b (http://www.relaxer.org)
 */
public class Label implements java.io.Serializable, Cloneable, IRVisitable, IRNode {
    private Boolean default_;
    // List<ILabelMixed>
    private java.util.List content_ = new java.util.ArrayList();
    private IRNode parentRNode_;

    /**
     * Creates a <code>Label</code>.
     *
     */
    public Label() {
    }

    /**
     * Creates a <code>Label</code>.
     *
     * @param source
     */
    public Label(Label source) {
        setup(source);
    }

    /**
     * Creates a <code>Label</code> by the Stack <code>stack</code>
     * that contains Elements.
     * This constructor is supposed to be used internally
     * by the Relaxer system.
     *
     * @param stack
     */
    public Label(RStack stack) {
        setup(stack);
    }

    /**
     * Creates a <code>Label</code> by the Document <code>doc</code>.
     *
     * @param doc
     */
    public Label(Document doc) {
        setup(doc.getDocumentElement());
    }

    /**
     * Creates a <code>Label</code> by the Element <code>element</code>.
     *
     * @param element
     */
    public Label(Element element) {
        setup(element);
    }

    /**
     * Creates a <code>Label</code> by the File <code>file</code>.
     *
     * @param file
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public Label(File file) throws IOException, SAXException, ParserConfigurationException {
        setup(file);
    }

    /**
     * Creates a <code>Label</code>
     * by the String representation of URI <code>uri</code>.
     *
     * @param uri
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public Label(String uri) throws IOException, SAXException, ParserConfigurationException {
        setup(uri);
    }

    /**
     * Creates a <code>Label</code> by the URL <code>url</code>.
     *
     * @param url
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public Label(URL url) throws IOException, SAXException, ParserConfigurationException {
        setup(url);
    }

    /**
     * Creates a <code>Label</code> by the InputStream <code>in</code>.
     *
     * @param in
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public Label(InputStream in) throws IOException, SAXException, ParserConfigurationException {
        setup(in);
    }

    /**
     * Creates a <code>Label</code> by the InputSource <code>is</code>.
     *
     * @param is
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public Label(InputSource is) throws IOException, SAXException, ParserConfigurationException {
        setup(is);
    }

    /**
     * Creates a <code>Label</code> by the Reader <code>reader</code>.
     *
     * @param reader
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public Label(Reader reader) throws IOException, SAXException, ParserConfigurationException {
        setup(reader);
    }

    /**
     * Initializes the <code>Label</code> by the Label <code>source</code>.
     *
     * @param source
     */
    public void setup(Label source) {
        int size;
        setDefault(source.getDefault());
        this.content_.clear();
        size = source.content_.size();
        for (int i = 0;i < size;i++) {
            addContent((ILabelMixed)source.getContent(i).clone());
        }
    }

    /**
     * Initializes the <code>Label</code> by the Document <code>doc</code>.
     *
     * @param doc
     */
    public void setup(Document doc) {
        setup(doc.getDocumentElement());
    }

    /**
     * Initializes the <code>Label</code> by the Element <code>element</code>.
     *
     * @param element
     */
    public void setup(Element element) {
        init(element);
    }

    /**
     * Initializes the <code>Label</code> by the Stack <code>stack</code>
     * that contains Elements.
     * This constructor is supposed to be used internally
     * by the Relaxer system.
     *
     * @param stack
     */
    public void setup(RStack stack) {
        init(stack.popElement());
    }

    /**
     * @param element
     */
    private void init(Element element) {
        RStack stack = new RStack(element);
        default_ = URelaxer.getAttributePropertyAsBooleanObject(element, "default");
        this.content_.clear();
        // XXX: mixed multi
        // URelaxer.isMatchDataValues(
        // URelaxer.isMatchDataComplex(
        while (true) {
            if (RString.isMatch(stack)) {
                addContent(new RString(stack));
            } else if (LabelType2.isMatch(stack)) {
                addContent(new LabelType2(stack));
            } else if (LabelType1.isMatch(stack)) {
                addContent(new LabelType1(stack));
            } else {
                break;
            }
        }
    }

    /**
     * @return Object
     */
    public Object clone() {
        return (new Label(this));
    }

    /**
     * Creates a DOM representation of the object.
     * Result is appended to the Node <code>parent</code>.
     *
     * @param parent
     */
    public void makeElement(Node parent) {
        Document doc;
        if (parent instanceof Document) {
            doc = (Document)parent;
        } else {
            doc = parent.getOwnerDocument();
        }
        Element element = doc.createElement("label");
        int size;
        if (this.default_ != null) {
            URelaxer.setAttributePropertyByBoolean(element, "default", this.default_);
        }
        size = this.content_.size();
        for (int i = 0;i < size;i++) {
            ILabelMixed value = (ILabelMixed)this.content_.get(i);
            value.makeElement(element);
        }
        parent.appendChild(element);
    }

    /**
     * Initializes the <code>Label</code> by the File <code>file</code>.
     *
     * @param file
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public void setup(File file) throws IOException, SAXException, ParserConfigurationException {
        setup(file.toURL());
    }

    /**
     * Initializes the <code>Label</code>
     * by the String representation of URI <code>uri</code>.
     *
     * @param uri
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public void setup(String uri) throws IOException, SAXException, ParserConfigurationException {
        setup(UJAXP.getDocument(uri, UJAXP.FLAG_NONE));
    }

    /**
     * Initializes the <code>Label</code> by the URL <code>url</code>.
     *
     * @param url
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public void setup(URL url) throws IOException, SAXException, ParserConfigurationException {
        setup(UJAXP.getDocument(url, UJAXP.FLAG_NONE));
    }

    /**
     * Initializes the <code>Label</code> by the InputStream <code>in</code>.
     *
     * @param in
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public void setup(InputStream in) throws IOException, SAXException, ParserConfigurationException {
        setup(UJAXP.getDocument(in, UJAXP.FLAG_NONE));
    }

    /**
     * Initializes the <code>Label</code> by the InputSource <code>is</code>.
     *
     * @param is
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public void setup(InputSource is) throws IOException, SAXException, ParserConfigurationException {
        setup(UJAXP.getDocument(is, UJAXP.FLAG_NONE));
    }

    /**
     * Initializes the <code>Label</code> by the Reader <code>reader</code>.
     *
     * @param reader
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public void setup(Reader reader) throws IOException, SAXException, ParserConfigurationException {
        setup(UJAXP.getDocument(reader, UJAXP.FLAG_NONE));
    }

    /**
     * Creates a DOM document representation of the object.
     *
     * @exception ParserConfigurationException
     * @return Document
     */
    public Document makeDocument() throws ParserConfigurationException {
        Document doc = UJAXP.makeDocument();
        makeElement(doc);
        return (doc);
    }

    /**
     * Sets a mixed content by <code>String</code>.
     *
     * @param value
     */
    public void setContent(String value) {
        setContent(new RString(value));
    }

    /**
     * Adds a mixed content by <code>String</code>.
     *
     * @param value
     */
    public void addContent(String value) {
        addContent(new RString(value));
    }

    /**
     * Sets a mixed content by <code>String</code>.
     *
     * @param value
     */
    public void setContentByString(String value) {
        setContent(new RString(value));
    }

    /**
     * Adds a mixed content by <code>String</code>.
     *
     * @param value
     */
    public void addContentByString(String value) {
        addContent(new RString(value));
    }

    /**
     * Sets a mixed content by <code>org.w3c.dom.Node</code>.
     *
     * @param value
     */
    public void setContent(org.w3c.dom.Node value) {
        setContent(new RString(value));
    }

    /**
     * Adds a mixed content by <code>org.w3c.dom.Node</code>.
     *
     * @param value
     */
    public void addContent(org.w3c.dom.Node value) {
        addContent(new RString(value));
    }

    /**
     * Gets the boolean property <b>default</b>.
     *
     * @return boolean
     */
    public boolean getDefault() {
        if (default_ == null) {
            return(false);
        }
        return (default_.booleanValue());
    }

    /**
     * Gets the boolean property <b>default</b>.
     *
     * @param defaultValue
     * @return boolean
     */
    public boolean getDefault(boolean defaultValue) {
        if (default_ == null) {
            return(defaultValue);
        }
        return (this.default_.booleanValue());
    }

    /**
     * Gets the boolean property <b>default</b>.
     *
     * @return Boolean
     */
    public Boolean getDefaultAsBoolean() {
        return (default_);
    }

    /**
     * Check the boolean property <b>default</b>.
     *
     * @return boolean
     */
    public boolean checkDefault() {
        return (default_ != null);
    }

    /**
     * Sets the boolean property <b>default</b>.
     *
     * @param defaultValue
     */
    public void setDefault(boolean defaultValue) {
        this.default_ = new Boolean(defaultValue);
    }

    /**
     * Sets the boolean property <b>default</b>.
     *
     * @param defaultValue
     */
    public void setDefault(Boolean defaultValue) {
        this.default_ = defaultValue;
    }

    /**
     * Gets the ILabelMixed property <b>content</b>.
     *
     * @return ILabelMixed[]
     */
    public ILabelMixed[] getContent() {
        ILabelMixed[] array = new ILabelMixed[content_.size()];
        return ((ILabelMixed[])content_.toArray(array));
    }

    /**
     * Sets the ILabelMixed property <b>content</b>.
     *
     * @param content
     */
    public void setContent(ILabelMixed[] content) {
        this.content_.clear();
        for (int i = 0;i < content.length;i++) {
            addContent(content[i]);
        }
        for (int i = 0;i < content.length;i++) {
            content[i].rSetParentRNode(this);
        }
    }

    /**
     * Sets the ILabelMixed property <b>content</b>.
     *
     * @param content
     */
    public void setContent(ILabelMixed content) {
        this.content_.clear();
        addContent(content);
        if (content != null) {
            content.rSetParentRNode(this);
        }
    }

    /**
     * Adds the ILabelMixed property <b>content</b>.
     *
     * @param content
     */
    public void addContent(ILabelMixed content) {
        this.content_.add(content);
        if (content != null) {
            content.rSetParentRNode(this);
        }
    }

    /**
     * Adds the ILabelMixed property <b>content</b>.
     *
     * @param content
     */
    public void addContent(ILabelMixed[] content) {
        for (int i = 0;i < content.length;i++) {
            addContent(content[i]);
        }
        for (int i = 0;i < content.length;i++) {
            content[i].rSetParentRNode(this);
        }
    }

    /**
     * Gets number of the ILabelMixed property <b>content</b>.
     *
     * @return int
     */
    public int sizeContent() {
        return (content_.size());
    }

    /**
     * Gets the ILabelMixed property <b>content</b> by index.
     *
     * @param index
     * @return ILabelMixed
     */
    public ILabelMixed getContent(int index) {
        return ((ILabelMixed)content_.get(index));
    }

    /**
     * Sets the ILabelMixed property <b>content</b> by index.
     *
     * @param index
     * @param content
     */
    public void setContent(int index, ILabelMixed content) {
        this.content_.set(index, content);
        if (content != null) {
            content.rSetParentRNode(this);
        }
    }

    /**
     * Adds the ILabelMixed property <b>content</b> by index.
     *
     * @param index
     * @param content
     */
    public void addContent(int index, ILabelMixed content) {
        this.content_.add(index, content);
        if (content != null) {
            content.rSetParentRNode(this);
        }
    }

    /**
     * Remove the ILabelMixed property <b>content</b> by index.
     *
     * @param index
     */
    public void removeContent(int index) {
        this.content_.remove(index);
    }

    /**
     * Remove the ILabelMixed property <b>content</b> by object.
     *
     * @param content
     */
    public void removeContent(ILabelMixed content) {
        this.content_.remove(content);
    }

    /**
     * Clear the ILabelMixed property <b>content</b>.
     *
     */
    public void clearContent() {
        this.content_.clear();
    }

    /**
     * Makes an XML text representation.
     *
     * @return String
     */
    public String makeTextDocument() {
        StringBuffer buffer = new StringBuffer();
        makeTextElement(buffer);
        return (new String(buffer));
    }

    /**
     * Makes an XML text representation.
     *
     * @param buffer
     */
    public void makeTextElement(StringBuffer buffer) {
        int size;
        buffer.append("<label");
        if (default_ != null) {
            buffer.append(" default=\"");
            buffer.append(URelaxer.getString(getDefault()));
            buffer.append("\"");
        }
        size = this.content_.size();
        for (int i = 0;i < size;i++) {
            ILabelMixed value = (ILabelMixed)this.content_.get(i);
            value.makeTextAttribute(buffer);
        }
        buffer.append(">");
        size = this.content_.size();
        for (int i = 0;i < size;i++) {
            ILabelMixed value = (ILabelMixed)this.content_.get(i);
            value.makeTextElement(buffer);
        }
        buffer.append("</label>");
    }

    /**
     * Makes an XML text representation.
     *
     * @param buffer
     * @exception IOException
     */
    public void makeTextElement(Writer buffer) throws IOException {
        int size;
        buffer.write("<label");
        if (default_ != null) {
            buffer.write(" default=\"");
            buffer.write(URelaxer.getString(getDefault()));
            buffer.write("\"");
        }
        size = this.content_.size();
        for (int i = 0;i < size;i++) {
            ILabelMixed value = (ILabelMixed)this.content_.get(i);
            value.makeTextAttribute(buffer);
        }
        buffer.write(">");
        size = this.content_.size();
        for (int i = 0;i < size;i++) {
            ILabelMixed value = (ILabelMixed)this.content_.get(i);
            value.makeTextElement(buffer);
        }
        buffer.write("</label>");
    }

    /**
     * Makes an XML text representation.
     *
     * @param buffer
     */
    public void makeTextElement(PrintWriter buffer) {
        int size;
        buffer.print("<label");
        if (default_ != null) {
            buffer.print(" default=\"");
            buffer.print(URelaxer.getString(getDefault()));
            buffer.print("\"");
        }
        size = this.content_.size();
        for (int i = 0;i < size;i++) {
            ILabelMixed value = (ILabelMixed)this.content_.get(i);
            value.makeTextAttribute(buffer);
        }
        buffer.print(">");
        size = this.content_.size();
        for (int i = 0;i < size;i++) {
            ILabelMixed value = (ILabelMixed)this.content_.get(i);
            value.makeTextElement(buffer);
        }
        buffer.print("</label>");
    }

    /**
     * Makes an XML text representation.
     *
     * @param buffer
     */
    public void makeTextAttribute(StringBuffer buffer) {
    }

    /**
     * Makes an XML text representation.
     *
     * @param buffer
     * @exception IOException
     */
    public void makeTextAttribute(Writer buffer) throws IOException {
    }

    /**
     * Makes an XML text representation.
     *
     * @param buffer
     */
    public void makeTextAttribute(PrintWriter buffer) {
    }

    /**
     * Gets the property value as String.
     *
     * @return String
     */
    public String getDefaultAsString() {
        return (URelaxer.getString(getDefault()));
    }

    /**
     * Sets the property value by String.
     *
     * @param string
     */
    public void setDefaultByString(String string) {
        setDefault(new Boolean(string).booleanValue());
    }

    /**
     * Returns a String representation of this object.
     * While this method informs as XML format representaion, 
     *  it's purpose is just information, not making 
     * a rigid XML documentation.
     *
     * @return String
     */
    public String toString() {
        try {
            return (makeTextDocument());
        } catch (Exception e) {
            return (super.toString());
        }
    }

    /**
     * Accepts the Visitor for enter behavior.
     *
     * @param visitor
     * @return boolean
     */
    public boolean enter(IRVisitor visitor) {
        return (visitor.enter(this));
    }

    /**
     * Accepts the Visitor for leave behavior.
     *
     * @param visitor
     */
    public void leave(IRVisitor visitor) {
        visitor.leave(this);
    }

    /**
     * Gets the IRNode property <b>parentRNode</b>.
     *
     * @return IRNode
     */
    public IRNode rGetParentRNode() {
        return (parentRNode_);
    }

    /**
     * Sets the IRNode property <b>parentRNode</b>.
     *
     * @param parentRNode
     */
    public void rSetParentRNode(IRNode parentRNode) {
        this.parentRNode_ = parentRNode;
    }

    /**
     * Gets child RNodes.
     *
     * @return IRNode[]
     */
    public IRNode[] rGetRNodes() {
        java.util.List classNodes = new java.util.ArrayList();
        classNodes.addAll(content_);
        IRNode[] nodes = new IRNode[classNodes.size()];
        return ((IRNode[])classNodes.toArray(nodes));
    }

    /**
     * Tests if a Element <code>element</code> is valid
     * for the <code>Label</code>.
     *
     * @param element
     * @return boolean
     */
    public static boolean isMatch(Element element) {
        if (!URelaxer.isTargetElement(element, "label")) {
            return (false);
        }
        RStack target = new RStack(element);
        boolean $match$ = false;
        Element child;
        if (RString.isMatch(target)) {
            $match$ = true;
        }
        if (LabelType2.isMatchHungry(target)) {
            $match$ = true;
        } else if (LabelType1.isMatchHungry(target)) {
            $match$ = true;
        } else {
        }
        if (!target.isEmptyElement()) {
            return (false);
        }
        return (true);
    }

    /**
     * Tests if elements contained in a Stack <code>stack</code>
     * is valid for the <code>Label</code>.
     * This mehtod is supposed to be used internally
     * by the Relaxer system.
     *
     * @param stack
     * @return boolean
     */
    public static boolean isMatch(RStack stack) {
        Element element = stack.peekElement();
        if (element == null) {
            return (false);
        }
        return (isMatch(element));
    }

    /**
     * Tests if elements contained in a Stack <code>stack</code>
     * is valid for the <code>Label</code>.
     * This method consumes the stack contents during matching operation.
     * This mehtod is supposed to be used internally
     * by the Relaxer system.
     *
     * @param stack
     * @return boolean
     */
    public static boolean isMatchHungry(RStack stack) {
        Element element = stack.peekElement();
        if (element == null) {
            return (false);
        }
        if (isMatch(element)) {
            stack.popElement();
            return (true);
        } else {
            return (false);
        }
    }
}
